! --- Subroutine generated on Fri Jul  1 07:24:47 CEST 2011

      subroutine fcnnut110701(mjd,X,Y,dX,dY)
      
!------------------------------------------------------------------------
!
!     Empirical model of FCN based on the IERS EOP 08 C 04 data
!
!     Input: mjd, modified julian date
!
!     Output: X, Y, CIP offsets (microas)
!             dX, dY, uncertainties on X, Y (microas)
!
!     Subroutine interface: none
!
!     Author: sebastien.lambert@obspm.fr
!
!------------------------------------------------------------------------

      implicit none
      
!     Input variables

      double precision mjd

!     Output variables
      
      double precision X,Y    ! FCN contributions
      double precision dX,dY  ! uncertainties on X, Y
      
!     Internal variables

      integer i,j,N
      parameter(N=28)
      double precision pi,per,phi,mpe
      double precision axc,axs,ayc,ays
      double precision daxc,daxs,dayc,days,dt,t
      double precision TABLE(4,N)
      double precision DATE(N),XC(N),XS(N),YC(N),YS(N),SX(N),SY(N)

      pi=3.14159265358979323846D0
      
!     Mean prediction error

      mpe=0.1325D0                       ! microas per day

!     FCN parameters

      per=-430.21D0                      ! period in days
      phi=(2.D0*pi/per)*(mjd-51544.5D0)  ! phase in rad
              
!     Block data of amplitudes for X (microas)
        
      data ((TABLE(i,j),i=1,4),j=1,N) /
     . 45700.D0,     4.55D0,   -36.58D0,    19.72D0, ! 1984.0
     . 46066.D0,  -141.82D0,  -105.35D0,    11.12D0, ! 1985.0
     . 46431.D0,  -246.56D0,  -170.21D0,     9.47D0, ! 1986.0
     . 46796.D0,  -281.89D0,  -159.24D0,     8.65D0, ! 1987.0
     . 47161.D0,  -255.05D0,   -43.58D0,     8.11D0, ! 1988.0
     . 47527.D0,  -210.46D0,   -88.56D0,     7.31D0, ! 1989.0
     . 47892.D0,  -187.79D0,   -57.35D0,     6.41D0, ! 1990.0
     . 48257.D0,  -163.01D0,    26.26D0,     5.52D0, ! 1991.0
     . 48622.D0,  -145.55D0,    44.64D0,     4.80D0, ! 1992.0
     . 48988.D0,  -146.73D0,    51.48D0,     6.02D0, ! 1993.0
     . 49353.D0,  -113.67D0,    13.06D0,     9.99D0, ! 1994.0
     . 49718.D0,   -87.14D0,     4.38D0,     8.68D0, ! 1995.0
     . 50083.D0,   -88.62D0,     3.15D0,     8.07D0, ! 1996.0
     . 50449.D0,   -95.52D0,    33.20D0,     4.44D0, ! 1997.0
     . 50814.D0,   -69.26D0,    26.92D0,     3.41D0, ! 1998.0
     . 51179.D0,   -43.92D0,   -14.58D0,     3.45D0, ! 1999.0
     . 51544.D0,     6.19D0,   -81.41D0,     3.22D0, ! 2000.0
     . 51910.D0,    69.27D0,  -133.07D0,     2.80D0, ! 2001.0
     . 52275.D0,    86.87D0,  -128.07D0,     2.69D0, ! 2002.0
     . 52640.D0,   111.42D0,   -43.33D0,     2.54D0, ! 2003.0
     . 53005.D0,   114.60D0,     0.21D0,     2.49D0, ! 2004.0
     . 53371.D0,   131.44D0,    -4.14D0,     2.68D0, ! 2005.0
     . 53736.D0,   155.41D0,    28.97D0,     2.16D0, ! 2006.0
     . 54101.D0,   158.77D0,    59.07D0,     1.84D0, ! 2007.0
     . 54466.D0,   155.49D0,   100.93D0,     1.72D0, ! 2008.0
     . 54832.D0,   142.30D0,   142.93D0,     1.88D0, ! 2009.0
     . 55197.D0,    35.98D0,   184.00D0,     1.96D0, ! 2010.0
     . 55562.D0,    23.45D0,   221.84D0,     2.08D0/ ! 2011.0
        
!     Amplitudes extracted from the table

      do i=1,N
         DATE(i)=TABLE(1,i)
         XC(i)=TABLE(2,i)
         XS(i)=TABLE(3,i)
         SX(i)=TABLE(4,i)
         YC(i)=XS(i)
         YS(i)=-XC(i)
         SY(i)=SX(i)
      end do

!     Prediction of the amplitude at the input date

      if (mjd.le.DATE(1)) then
         axc=XC(1)
         axs=XS(1)
         ayc=YC(1)
         ays=YS(1)
      else if (mjd.ge.DATE(N)) then
         axc=XC(N)
         axs=XS(N)
         ayc=YC(N)
         ays=YS(N)
      else
         do i=1,N-1
            if (mjd.ge.DATE(i).and.mjd.lt.DATE(i+1)) then
               t=mjd-DATE(i)
               dt=DATE(i+1)-DATE(i)
               daxc=XC(i+1)-XC(i)
               daxs=XS(i+1)-XS(i)
               dayc=YC(i+1)-YC(i)
               days=YS(i+1)-YS(i)
               axc=XC(i)+(daxc/dt)*t
               axs=XS(i)+(daxs/dt)*t
               ayc=YC(i)+(dayc/dt)*t
               ays=YS(i)+(days/dt)*t
            end if
         end do
      end if
      
!     Computation of X and Y

      X=axc*dcos(phi)-axs*dsin(phi)  ! microas
      Y=ayc*dcos(phi)-ays*dsin(phi)  ! microas
      
!     Prediction of the uncertainty at the input date

      if (mjd.le.DATE(1)) then
         axc=SX(1)+mpe*(DATE(1)-mjd)
         axs=SX(1)+mpe*(DATE(1)-mjd)
         ayc=SY(1)+mpe*(DATE(1)-mjd)
         ays=SY(1)+mpe*(DATE(1)-mjd)
      else if (mjd.ge.DATE(N)) then
         axc=SX(N)+mpe*(mjd-DATE(N))
         axs=SX(N)+mpe*(mjd-DATE(N))
         ayc=SY(N)+mpe*(mjd-DATE(N))
         ays=SY(N)+mpe*(mjd-DATE(N))
      else
         do i=1,N-1
            if (mjd.ge.DATE(i).and.mjd.lt.DATE(i+1)) then
               t=mjd-DATE(i)
               dt=DATE(i+1)-DATE(i)
               daxc=SX(i+1)-SX(i)
               daxs=SX(i+1)-SX(i)
               dayc=SY(i+1)-SY(i)
               days=SY(i+1)-SY(i)
               axc=dabs(SX(i)+(daxc/dt)*t)
               axs=dabs(SX(i)+(daxs/dt)*t)
               ayc=dabs(SY(i)+(dayc/dt)*t)
               ays=dabs(SY(i)+(days/dt)*t)
            end if
         end do
      end if
      
!     Computation of the uncertainties

      dX=axc+axs  ! microas
      dY=ayc+ays  ! microas
      
      end
