      program time_asc2eph
      implicit none
C       n.b. this programme is a modification
C       (by Alan W. Irwin, irwin@uvastro.phys.uvic.ca)
C       of the JPL asc2eph that is only
C       suitable for creation of a binary time ephemeris file.
C
C      time_asc2eph creates a direct access Irwin/Fukushima Time
C       Ephemeris file from
C       a header ascii text file and data ascii text file.
C
C      This program, 'time_asc2eph', requires (via standard input) an ascii
C      header file ('time_header.XXX'), concatanated with an ascii time ephemeris 
C      data file ('time_asc.XXX').  Both files must have the same ephemeris
C      number, XXX (e.g., 405).
C
C      By default, the output ephemeris will span the same interval as the input
C      text file(s).  If you are interested in only a portion of data, set the
C      below T1 and T2 to the begin and end times of the span you desire.  T1
C      and T2 must be specified in  Julian Ephemeris Days (ET).
C
C      A sample execution under unix/linux is:
C
C       cat  time_header.405  time_asc.405 | time_asc2eph
C
C **********************************************************************************
C
C                                    *** NOTE ***
C
C      However, the units in which the length of a direct access record is specified 
C      are PROCESSOR DEPENDENT.  The parameter NRECL, the number of units per word, 
C      controls the length of a record in the direct access ephemeris.
C
C *****  The user must choose one of the following statements  *****
C
      integer*4 nrecl
C       unix/linux choice
       PARAMETER ( NRECL = 4 )
c       PARAMETER ( NRECL = 1 )

C **********************************************************************************
       integer*4 nconmax
       parameter(nconmax = 2)
       CHARACTER*6  CNAM (nconmax)
       CHARACTER*6  TTL  (14,3)
       CHARACTER*12 HEADER

!       DOUBLE PRECISION  AU
       DOUBLE PRECISION  CVAL (nconmax)
       DOUBLE PRECISION  DB   (3000), DB2Z
C        initialize this variable to JD before any conceivable JD
       data db2z/-1.d30/
!       DOUBLE PRECISION  EMRAT
       DOUBLE PRECISION  SS   (3)
       DOUBLE PRECISION  T1, T2

       integer*4 jmax
       parameter(jmax = 2)
       INTEGER  I, IPT (3,jmax), IRECSZ, IN
       INTEGER  J
       INTEGER  K,KSIZE
!       INTEGER  LPT (3)
       INTEGER  N, NCON, NROUT, NCOEFF, NRW
!       integer*4 NUMDE
       INTEGER  OUT

       LOGICAL  FIRST
       DATA     FIRST / .TRUE. /

C ***********************************************************************
C     By default, the output ephemeris will span the same interval as the
C     input ascii data file(s).  The user may reset these to other JED's.
C
       T1 = 0.D0
       T2 = 9999999.d0


      if(nrecl .eq. 0) then
      write(*,*)'*** ERROR: User did not set NRECL ***'
      stop
      endif


C      Write a fingerprint to the screen.

       WRITE(*,*) ' Irwin ASCII-TO-DIRECT-I/O program. ' //
     &   ' Last modified Oct-1998.'


C
C      Read the size and number of main ephemeris records.
C
       READ  (*,'(6X,I6)')  KSIZE
       WRITE (*,100)              KSIZE
100    FORMAT(/'KSIZE =',I6)

       IRECSZ = NRECL * KSIZE


C
C      Now for the alphameric heading records (GROUP 1010)
C
       CALL  NXTGRP ( HEADER )

       IF (HEADER .NE. 'GROUP   1010')  THEN
          CALL  ERRPRT ( 1010, 'NOT HEADER' )
       ENDIF

       READ (*,'(14A6)')    TTL
       WRITE(*,'(/(14A6))') TTL


C
C      Read start, end and record span  (GROUP 1030)
C
       CALL  NXTGRP ( HEADER )

       IF ( HEADER .NE. 'GROUP   1030' ) THEN
          CALL ERRPRT ( 1030, 'NOT HEADER' )
       ENDIF

       READ (*,'(3D12.0)')  SS


C
C      Read number of constants and names of constants (GROUP 1040/4).
C
       CALL  NXTGRP ( HEADER )

       IF ( HEADER .NE. 'GROUP   1040 ') THEN
          CALL ERRPRT ( 1040, 'NOT HEADER' )
       ENDIF

       READ (*,'(I6)')    N
       NCON = N
       if(ncon.gt.nconmax) stop 'too many constants'
       READ (*,'(10A8)') (CNAM(I),I=1,N)



C
C      Read number of values and values (GROUP 1041/4)
C
       CALL  NXTGRP ( HEADER )

       IF ( HEADER .NE. 'GROUP   1041' ) THEN
          CALL ERRPRT ( 1041, 'NOT HEADER' )
       ENDIF

       READ (*,'(I6)')       N
       if(n.ne.ncon) stop 'inconsistent number of constants'
       READ (*,'(3D26.18)')  (CVAL(I),I=1,N)

!       DO  I = 1, N
!           IF ( CNAM(I) .EQ. 'AU    ' )  AU    = CVAL(I)
!           IF ( CNAM(I) .EQ. 'EMRAT ' )  EMRAT = CVAL(I)
!           IF ( CNAM(I) .EQ. 'DENUM ' )  NUMDE = CVAL(I)
!       END DO

       WRITE (*,'(500(/2(A8,D24.16)))') (CNAM(I),CVAL(I),I=1,N)


C
C      Read pointers needed by INTERP (GROUP 1050)
C
       CALL  NXTGRP ( HEADER )

       IF ( HEADER .NE. 'GROUP   1050' ) THEN
          CALL ERRPRT ( 1050, 'NOT HEADER' )
       ENDIF

       READ (*,'(2I6)')   ((IPT(I,J),J=1,jmax),I=1,3)
       WRITE(*,'(/(3I5))') IPT


C
C      Open direct-access output file ('JPLEPH')
C
       OPEN ( UNIT   = 12,
     .        FILE   = 'TIMEEPH',
     .        ACCESS = 'DIRECT',
     .        FORM   = 'UNFORMATTED',
     .        RECL   = IRECSZ,
     .        STATUS = 'NEW' )


C
C     Read and write the ephemeris data records (GROUP 1070).
C
      CALL  NXTGRP ( HEADER )

      IF ( HEADER .NE. 'GROUP   1070' ) CALL ERRPRT(1070,'NOT HEADER')

      NROUT  = 0
      IN     = 0
      OUT    = 0

      READ (*,'(2I6,/(3D26.18))',IOSTAT =IN)
     &  NRW, NCOEFF, (DB(K),K=1,NCOEFF)


      DO WHILE (       ( IN    .EQ. 0 )
     .           .AND. ( DB(2) .LT. T2) )

          IF ( 2*NCOEFF .NE. KSIZE ) THEN
             CALL ERRPRT(NCOEFF,' 2*NCOEFF not equal to KSIZE')
          ENDIF

C
C         Skip this data block if the end of the interval is less
C         than the specified start time or if it begins before
C         where the previous block ended.
C
          IF  ( (DB(2) .GE. T1) .AND. (DB(1) .GE. DB2Z) ) THEN

             IF ( FIRST ) THEN
C
C               Don't worry about the intervals overlapping
C               or abutting if this is the first applicable
C               interval.
C
                DB2Z  = DB(1)
                FIRST = .FALSE.
             ENDIF

             IF (DB(1) .NE. DB2Z ) THEN
C 
C               Beginning of current interval is past the end
C               of the previous one.
C
                CALL ERRPRT (NRW, 'Records do not overlap or abut')
             ENDIF

             DB2Z  = DB(2)
             NROUT = NROUT + 1

             WRITE (12,REC=NROUT+2,IOSTAT=OUT) (DB(K),K=1,NCOEFF)

             IF ( OUT .NE. 0 ) THEN
                CALL ERRPRT (NROUT, 
     .                     'th record not written because of error')
             ENDIF

C
C            Save this block's starting date, its interval span, and its end 
C            date.
C
             IF (NROUT .EQ. 1) THEN
                SS(1) = DB(1)
                SS(3) = DB(2) - DB(1)
             ENDIF
             SS(2) = DB(2)

C
C            Update the user as to our progress every 100th block.
C
             IF ( MOD(NROUT,100) .EQ. 1 ) THEN
                IF ( DB(1) .GE. T1 ) THEN
                   WRITE (*,271) NROUT, DB(2)
                ELSE
                   WRITE (*,*) 
     .           ' Searching for first requested record...'
                ENDIF
             ENDIF
271          FORMAT (I6,
     .              ' EPHEMERIS RECORDS WRITTEN.  LAST JED = ',
     .              F12.2)

          ENDIF

          READ (*,'(2I6,/(3D26.18))',IOSTAT =IN)
     &      NRW, NCOEFF, (DB(K),K=1,NCOEFF)

       END DO

       WRITE (*,275) NROUT, DB(2)
275    FORMAT(/I6, ' EPHEMERIS RECORDS WRITTEN.  LAST JED = ',F12.2)


C
C      Write header records onto output file.
C
       NROUT = 1
!       WRITE(12,REC=1,IOSTAT=OUT) TTL,CNAM,SS,NCON,AU,
!     .                            EMRAT,IPT,NUMDE,LPT
       WRITE(12,REC=1,IOSTAT=OUT) TTL,CNAM,SS,NCON,IPT
       IF ( OUT .NE. 0 ) THEN
          CALL ERRPRT ( NROUT, 'th record not written because of error')
       ENDIF

       NROUT = 2
       WRITE(12,REC=2,IOSTAT=OUT)CVAL
       IF ( OUT .NE. 0 ) THEN
          CALL ERRPRT ( NROUT, 'th record not written because of error')
       ENDIF


C
C      We're through.  Wrap it up.
C
       CLOSE (12)
       STOP ' OK'


       END

C      $Header: time_asc2eph.oth,v 1.1 93/08/23 14:53:39 faith Exp $
C
       SUBROUTINE  ERRPRT (I, MSG)

       CHARACTER*(*)  MSG
       INTEGER        I

       WRITE (*,200)  I, MSG
200    FORMAT('ERROR #',I8,2X,A50)

       STOP ' ERROR '
       END


      SUBROUTINE  NXTGRP ( HEADER )

      CHARACTER*(*)  HEADER
      CHARACTER*12   BLANK

C
C     Start with nothing.
C
      HEADER = ' '

C
C     The next non-blank line we encounter is a header record.
C     The group header and data are seperated by a blank line.
C
      DO WHILE ( HEADER .EQ. ' ' )
          READ (*,'(A)') HEADER
      ENDDO

C     Found the header.  Read the blank line so we can get at the data.
      READ (*, '(A)') BLANK

      RETURN
      END
